=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2006-06-15 21:16:20 +0000
+++ bzrlib/tests/stub_sftp.py	2006-06-29 04:08:32 +0000
@@ -22,6 +22,7 @@
 import os
 from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
     SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
+import sys
 
 from bzrlib.osutils import pathjoin
 from bzrlib.trace import mutter
@@ -64,17 +65,40 @@
 
     def __init__(self, server, root, home=None):
         SFTPServerInterface.__init__(self, server)
+        # All paths are actually relative to 'root'.
+        # this is like implementing chroot().
         self.root = root
         if home is None:
+            # XXX: if 'home' is None, shouldn't it
+            #       be set to '', since it should
+            #       be relative to 'root'?
             self.home = self.root
         else:
+            assert home.startswith(self.root), \
+                    "home must be a subdirectory of root (%s vs %s)" \
+                    % (home, root)
             self.home = home[len(self.root):]
-        if (len(self.home) > 0) and (self.home[0] == '/'):
+        if self.home.startswith('/'):
             self.home = self.home[1:]
         server._test_case.log('sftpserver - new connection')
 
     def _realpath(self, path):
-        return self.root + self.canonicalize(path)
+        if sys.platform == 'win32':
+            # Win32 sftp paths end up looking like
+            # sftp://host@foo/h:/foo/bar
+            # which gets translated here to:
+            # /h:/foo/bar
+            # Local paths stay 'foo/bar', though.
+            # Also, win32 needs to use the Unicode APIs.
+            thispath = path.decode('utf8')
+            if path.startswith('/'):
+                # Abspath
+                realpath = os.path.normpath(thispath[1:])
+            else:
+                realpath = os.path.normpath(os.path.join(self.home, thispath))
+        else:
+            realpath = self.root + self.canonicalize(path)
+        return realpath
 
     def canonicalize(self, path):
         if os.path.isabs(path):
@@ -96,7 +120,10 @@
             # TODO: win32 incorrectly lists paths with non-ascii if path is not
             # unicode. However on Linux the server should only deal with
             # bytestreams and posix.listdir does the right thing 
-            flist = os.listdir(path)
+            if sys.platform == 'win32':
+                flist = [f.encode('utf8') for f in os.listdir(path)]
+            else:
+                flist = os.listdir(path)
             for fname in flist:
                 attr = SFTPAttributes.from_stat(os.stat(pathjoin(path, fname)))
                 attr.filename = fname

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-06-18 15:53:41 +0000
+++ bzrlib/transport/sftp.py	2006-06-29 04:08:32 +0000
@@ -40,7 +40,7 @@
                            PathError,
                            ParamikoNotPresent,
                            )
-from bzrlib.osutils import pathjoin, fancy_rename
+from bzrlib.osutils import pathjoin, fancy_rename, getcwd
 from bzrlib.trace import mutter, warning, error
 from bzrlib.transport import (
     register_urlparse_netloc_protocol,
@@ -375,6 +375,7 @@
                 basepath.append(p)
 
         path = '/'.join(basepath)
+        # mutter('relpath => remotepath %s => %s', relpath, path)
         return path
 
     def relpath(self, abspath):
@@ -498,8 +499,8 @@
 
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
+        path = self._remote_path(relpath)
         try:
-            path = self._remote_path(relpath)
             # In the paramiko documentation, it says that passing a mode flag 
             # will filtered against the server umask.
             # StubSFTPServer does not do this, which would be nice, because it is
@@ -850,6 +851,7 @@
         :param mode: The mode permissions bits for the new file
         """
         path = self._sftp._adjust_cwd(abspath)
+        # mutter('sftp abspath %s => %s', abspath, path)
         attr = SFTPAttributes()
         if mode is not None:
             attr.st_mode = mode
@@ -959,7 +961,7 @@
 
     def _run_server(self, s):
         ssh_server = paramiko.Transport(s)
-        key_file = os.path.join(self._homedir, 'test_rsa.key')
+        key_file = pathjoin(self._homedir, 'test_rsa.key')
         f = open(key_file, 'w')
         f.write(STUB_SERVER_KEY)
         f.close()
@@ -977,11 +979,12 @@
         global _ssh_vendor
         self._original_vendor = _ssh_vendor
         _ssh_vendor = self._vendor
-        self._homedir = os.getcwd()
+        self._homedir = getcwd()
         if self._server_homedir is None:
             self._server_homedir = self._homedir
         self._root = '/'
-        # FIXME WINDOWS: _root should be _server_homedir[0]:/
+        if sys.platform == 'win32':
+            self._root = ''
         self._listener = SocketListener(self._run_server)
         self._listener.setDaemon(True)
         self._listener.start()
@@ -1039,7 +1042,10 @@
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""
-        return self._get_sftp_url(urlutils.escape(self._homedir[1:]))
+        if sys.platform == 'win32':
+            return self._get_sftp_url(urlutils.escape(self._homedir))
+        else:
+            return self._get_sftp_url(urlutils.escape(self._homedir[1:]))
 
 
 class SFTPHomeDirServer(SFTPServerWithoutSSH):

